import os
import threading

import uuid
from pathlib import Path
from queue import Queue

from slicing.stmt.statement import Statement
from slicing.stype.line_number import LineNumber


class Writer(threading.Thread):
    def __init__(self, slice_ins, line_number: LineNumber):
        super().__init__()
        self._slice_ins = slice_ins
        self.__out_path_folder = Path(slice_ins.out_put).joinpath(slice_ins.task_id)
        self.sql_statements = Queue()
        # self._finish = False
        self.stop_event = threading.Event()
        self.line_number = line_number

    def add_statements(self, statements: list[bytes]):
        self.sql_statements.put(statements)

    def write_file(self, sql_statement_bytes: list[bytes]):
        sql_statement_byte = sql_statement_bytes[0]
        sql_statement = sql_statement_byte[0:self._range_index(sql_statement_byte)].decode()
        statement = Statement(sql_statement)
        statement.visitor()
        if not statement.ignore():
            self.line_number.add()
            folder = self.create_folder(statement.operate())
            table_name = statement.table()

            file_id = str(uuid.uuid4()).replace("-", "")
            file_name = f'{table_name}-{file_id}'
            with open(folder.joinpath(Path(f'{file_name}.sql')), 'ab') as f:
                if statement.operate().upper() == "CREATE":
                    f.write(f'DROP TABLE IF EXISTS `{table_name}`;\n'.encode("UTF-8"))
                for sql_bytes in sql_statement_bytes:
                    f.write(sql_bytes)
            size = os.path.getsize(folder.joinpath(Path(f'{file_name}.sql')))
            self._slice_ins.file_list_writer.add_sql(file_id, statement, size, self.line_number)

    def create_folder(self, operate):
        path = self.__out_path_folder.joinpath(operate)
        if not path.exists():
            path.mkdir(parents=True, exist_ok=True)
        return path

    def run(self):
        try:
            while not self.stop_event.is_set():
                # if self.sql_statements.qsize() > 0:
                sql_statement_bytes = self.sql_statements.get()
                if sql_statement_bytes == "Thread Stop": break
                self.write_file(sql_statement_bytes)

        except Exception as e:
            raise e

    def finish(self):
        self.sql_statements.put("Thread Stop")

    def stop(self):
        self.stop_event.set()
        self.finish()

    def _range_index(self, line) -> int:
        def count(i):
            return i if i != -1 else None

        if line.startswith(b'INSERT') or line.startswith(b'insert') or line.startswith(b'Insert'):
            index = count(line.find(b"VALUES")) or count(line.find(b"values")) or count(line.find(b"Values"))
            if index: return index
        return 200
